# -*- coding: utf-8 -*-
import os

from .context import Context
from .error import CoppyrError


__title__ = "coppyr"
__summary__ = "A collection of boilerplate for Python applications."
__url__ = "https://github.com/gshulegaard/coppyr"

__version__ = "0.9.3"

__author__ = "Grant Hulegaard"
__email__ = "loki.labrys@gmail.com"

__license__ = "MIT"
__copyright__ = "Copyright (c) 2019 Grant Hulegaard"


here = os.path.dirname(__file__)
